"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var crypto = require("crypto");
var fileType = require("file-type");
var path = require("path");
var postcss = require("postcss");
var request = require("request-promise");
var chalk = require('chalk');
var urlReg = /url\(('|")?((?:http|\/\/)(?:[^"']+))(\1)\)/;
var getDeclUrl = function (value) {
    var url = value.match(urlReg)[2];
    var md5 = crypto.createHash('md5');
    var urlIdentity = md5.update(url).digest('hex');
    return { urlIdentity: urlIdentity, url: url };
};
// @ts-ignore
exports.default = postcss.plugin('postcss-assets', function (_a, opts) {
    var outputOptions = _a.outputOptions, options = _a.options;
    if (opts === void 0) { opts = {}; }
    // 所有 css 中的网络请求
    var networkRequestMap = {};
    var isForceLocal = options.forceLocal || false;
    var publicPath = outputOptions.publicPath || '';
    var isUrlPublicPath = /^(https?:)?\/\//.test(publicPath);
    return function (root) {
        return new Promise(function (resolve) {
            // 字体文件
            root.walkAtRules(function (atrule) {
                atrule.walkDecls(function (decl) {
                    if (decl.prop === 'src') {
                        decl.value.split(',').forEach(function (value) {
                            if (urlReg.test(value)) {
                                var _a = getDeclUrl(value), url = _a.url, urlIdentity = _a.urlIdentity;
                                if (!isForceLocal && isUrlPublicPath && url.startsWith(publicPath)) {
                                    // 已经是 publicPath 名下的网络资源可以不用本地化
                                    return;
                                }
                                networkRequestMap[urlIdentity] = { url: url, decl: decl };
                            }
                        });
                    }
                });
            });
            // 常规 css
            root.walkRules(function (rule) {
                rule.walkDecls(function (decl) {
                    if (decl.prop === 'background-image' || decl.prop === 'background') {
                        if (urlReg.test(decl.value)) {
                            var _a = getDeclUrl(decl.value), url = _a.url, urlIdentity = _a.urlIdentity;
                            if (!isForceLocal && isUrlPublicPath && url.startsWith(publicPath)) {
                                // 已经是 publicPath 名下的网络资源可以不用本地化
                                return;
                            }
                            networkRequestMap[urlIdentity] = { url: url, decl: decl };
                        }
                    }
                });
            });
            if (Object.keys(networkRequestMap).length > 0) {
                Promise.all(Object.entries(networkRequestMap).map(function (_a) {
                    var urlIdentity = _a[0], networkRequest = _a[1];
                    var originUrl = networkRequest.url;
                    var url = originUrl.startsWith('http')
                        ? originUrl
                        : "http:".concat(originUrl);
                    return request
                        .get(__assign({ url: url, encoding: null }, options.requsetOptions))
                        .then(function (res) {
                        var buffer = Buffer.from(res, 'utf-8');
                        var fileExtName = path.extname(url);
                        var fileExtType = fileType(buffer);
                        var md5 = crypto.createHash('md5');
                        var ext = fileExtType && fileExtType.ext
                            ? ".".concat(fileExtType.ext)
                            : fileExtName;
                        var basename = md5.update(buffer).digest('hex') + ext;
                        var contextPath = path
                            .join(options.relativeCssPath, options.outputPath, basename)
                            .replace(/\\/g, '/');
                        var outputPath = path
                            .join(options.outputPath, basename)
                            .replace(/\\/g, '/');
                        var publicFullPath = "".concat(outputOptions.publicPath).concat(outputPath);
                        var asset = {
                            contents: buffer,
                            contextPath: contextPath,
                            outputPath: outputPath,
                            basename: basename,
                            publicFullPath: publicFullPath,
                        };
                        networkRequestMap[urlIdentity] = asset;
                        opts.emit(asset);
                        return Promise.resolve(asset);
                    })
                        .catch(function (err) {
                        console.log(chalk.cyan('[ExtractCssAssetsWebpackPlugin]'), chalk.yellow('Warning:'), 'Asset download failed', chalk.blue.underline(url));
                        console.log('   ', err.error.toString());
                        delete networkRequestMap[urlIdentity];
                    });
                })).then(function () {
                    // 字体文件
                    root.walkAtRules(function (atrule) {
                        atrule.walkDecls(function (decl) {
                            if (decl.prop === 'src') {
                                var newValue = decl.value
                                    .split(',')
                                    .map(function (value) {
                                    if (urlReg.test(value)) {
                                        var _a = getDeclUrl(value), urlIdentity_1 = _a.urlIdentity, url = _a.url;
                                        if (!isForceLocal && isUrlPublicPath && url.startsWith(publicPath)) {
                                            // 已经是 publicPath 名下的网络资源可以不用本地化
                                            return value;
                                        }
                                        return value.replace(urlReg, function (str) {
                                            if (networkRequestMap[urlIdentity_1] &&
                                                isUrlPublicPath &&
                                                networkRequestMap[urlIdentity_1].publicFullPath) {
                                                return "url('".concat(networkRequestMap[urlIdentity_1].publicFullPath, "')");
                                            }
                                            else if (networkRequestMap[urlIdentity_1] &&
                                                networkRequestMap[urlIdentity_1].contextPath) {
                                                return "url('".concat(networkRequestMap[urlIdentity_1].contextPath, "')");
                                            }
                                            return str;
                                        });
                                    }
                                    return value;
                                })
                                    .join(',');
                                decl.value = newValue;
                            }
                        });
                    });
                    // 常规 css
                    root.walkRules(function (rule) {
                        rule.walkDecls(function (decl) {
                            if (decl.prop === 'background-image' ||
                                decl.prop === 'background') {
                                if (urlReg.test(decl.value)) {
                                    var _a = getDeclUrl(decl.value), urlIdentity_2 = _a.urlIdentity, url = _a.url;
                                    if (!isForceLocal && isUrlPublicPath && url.startsWith(publicPath)) {
                                        // 已经是 publicPath 名下的网络资源可以不用本地化
                                        return;
                                    }
                                    decl.value = decl.value.replace(urlReg, function (str) {
                                        if (networkRequestMap[urlIdentity_2] &&
                                            isUrlPublicPath &&
                                            networkRequestMap[urlIdentity_2].publicFullPath) {
                                            return "url('".concat(networkRequestMap[urlIdentity_2].publicFullPath, "')");
                                        }
                                        else if (networkRequestMap[urlIdentity_2] &&
                                            networkRequestMap[urlIdentity_2].contextPath) {
                                            return "url('".concat(networkRequestMap[urlIdentity_2].contextPath, "')");
                                        }
                                        return str;
                                    });
                                }
                            }
                        });
                    });
                    resolve();
                });
            }
            else {
                resolve();
            }
        });
    };
});
//# sourceMappingURL=process-assets.js.map